//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UnitMain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormMain *FormMain;


#include "WaveFile.h"

#define TITLE	"SampleCut v1.0"

WaveFile* WaveSource;

typedef struct {
	int start;
	int end;
} WAVE_SEGMENT;

#define MAX_SEGMENTS	65536

WAVE_SEGMENT WaveSegment[MAX_SEGMENTS];

int WaveSegmentCount;


void __fastcall TFormMain::RedrawSampleView(void)
{
	Graphics::TBitmap* bmp;
	TCanvas* c;
	int n,sx,sp,col,max,pps,spp,sgptr;
	long ptr;
	const int colors[4]={0x004000,0x004040,0x404000,0x400040};

	bmp=ImageSampleView->Picture->Bitmap;
	c=bmp->Canvas;

	c->Lock();

	pps=TrackBarScale->Position*100;		//pixels per second
	spp=WaveSource->GetSampleRate()/pps;	//samples per pixel
	max=WaveSource->GetLength()-bmp->Width*spp;
	ptr=max/1024*ScrollBarView->Position;	//viewport offset
	sgptr=0;

	if(max<0) max=0;
	if(ptr<0) ptr=0;
	if(ptr>max) ptr=max;

	for(sx=0;sx<bmp->Width;++sx)
	{
		col=0x000000;

		while((WaveSegment[sgptr].end<ptr)&&(sgptr<WaveSegmentCount)) ++sgptr;

		if(sgptr<WaveSegmentCount)
		{
			if(ptr>=WaveSegment[sgptr].start&&ptr<WaveSegment[sgptr].end) col=colors[sgptr&3];
		}
		
		if(((ptr/spp)%(pps/10))==0) col+=0x101010;
		if(((ptr/spp)%pps)==0) col+=0x202020;

		c->Brush->Color=(TColor)col;
		c->FillRect(TRect(sx,0,sx+1,bmp->Height));

		n=0;

		for(sp=0;sp<spp;++sp)
		{
			n+=abs(WaveSource->GetSample(ptr));
			++ptr;
		}

		n=n/spp*bmp->Height/65536;

		c->Brush->Color=clLime;
		c->FillRect(TRect(sx,bmp->Height/2-n,sx+1,bmp->Height/2+n));
	}

	c->Brush->Color=clBlack;
	c->Font->Color=clWhite;
	c->Font->Size=12;
	c->TextOut(10,10,"Segments: "+IntToStr(WaveSegmentCount));

	c->Unlock();
}



void __fastcall TFormMain::Analyze(void)
{
	int i,n,s,ptr,gate,sgptr,sample_len,stage;
	int frame_len=WaveSource->GetSampleRate()/1000;
	int atk_threshold=TrackBarAttack->Position;
	int rel_threshold=TrackBarRelease->Position;
	int rel_speed=TrackBarRelSpeed->Position,rel_time_cnt;

	sample_len=WaveSource->GetLength();

	gate=TrackBarGate->Position;
	sgptr=0;
	stage=0;

	for(ptr=0;ptr<sample_len;ptr+=frame_len)
	{
		n=0;

		for(i=0;i<frame_len*2;++i)
		{
			if(ptr+i<sample_len)
			{
				s=abs(WaveSource->GetSample(ptr+i));
				if(s<gate) s=0;
				n+=s;
			}
		}

		if(stage==0&&n>=atk_threshold)
		{
			stage=1;
			WaveSegment[sgptr].start=ptr;
			continue;
		}

		if(stage==1)
		{
			if(n<rel_threshold)
			{
				--rel_time_cnt;

				if(rel_time_cnt<=0)
				{
					stage=0;
					WaveSegment[sgptr++].end=ptr;

					if(sgptr>=MAX_SEGMENTS) break;
				}
			}
			else
			{
				rel_time_cnt=rel_speed;
            }
		}
	}

	WaveSegmentCount=sgptr;
}
//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SpeedButtonLoadClick(TObject *Sender)
{
	if(!OpenDialogWAV->Execute()) return;

	WaveSource->Load(OpenDialogWAV->FileName);

	memset(WaveSegment,0,sizeof(WaveSegment));
	WaveSegmentCount=0;

	Analyze();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::SpeedButtonSaveClick(TObject *Sender)
{
	AnsiString filename;
	int i;

	if(!SaveDialogWAV->Execute()) return;

	Analyze();

	for(i=0;i<WaveSegmentCount;++i)
	{
		 filename=filename.Format("%s_%2.2u.wav",ARRAYOFCONST((ChangeFileExt(SaveDialogWAV->FileName,""),i)));

		 Caption="Saving "+filename+"...";
		 
		 WaveSource->SaveSelection(filename,WaveSegment[i].start,WaveSegment[i].end);
	}

	Caption=TITLE;
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::FormCreate(TObject *Sender)
{
	ImageSampleView->Picture=new TPicture();
	ImageSampleView->Picture->Bitmap=new Graphics::TBitmap();
	ImageSampleView->Picture->Bitmap->PixelFormat=pf24bit;
	ImageSampleView->Picture->Bitmap->SetSize(ImageSampleView->Width,ImageSampleView->Height);
	
	DoubleBuffered=true;
	PanelView->DoubleBuffered=true;
	
	WaveSource=new WaveFile();

	memset(WaveSegment,0,sizeof(WaveSegment));
	WaveSegmentCount=0;

	Caption=TITLE;
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::FormDestroy(TObject *Sender)
{
	delete WaveSource;
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::Timer1Timer(TObject *Sender)
{
	RedrawSampleView();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::PanelViewResize(TObject *Sender)
{
	ImageSampleView->Picture->Bitmap->SetSize(ImageSampleView->Width,ImageSampleView->Height);
	RedrawSampleView();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::FormCanResize(TObject *Sender, int &NewWidth,
int &NewHeight, bool &Resize)
{
	NewHeight=Height;
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarAttackChange(TObject *Sender)
{
	TrackBarAttack->Hint=IntToStr(TrackBarAttack->Position);
	Analyze();
}
//---------------------------------------------------------------------------


void __fastcall TFormMain::TrackBarGateChange(TObject *Sender)
{
	TrackBarGate->Hint=IntToStr(TrackBarGate->Position);
	Analyze();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarReleaseChange(TObject *Sender)
{
	TrackBarRelease->Hint=IntToStr(TrackBarRelease->Position);
	Analyze();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarRelSpeedChange(TObject *Sender)
{
	TrackBarRelSpeed->Hint=IntToStr(TrackBarRelSpeed->Position);
	Analyze();
}
//---------------------------------------------------------------------------

