#include "WaveFile.h"


WaveFile::WaveFile()
{
	wData=NULL;
	wSize=0;
	wOffset=0;
	wChannels=1;
	wBits=16;
	wSampleRate=44100;
	wLengthSamples=0;
}



WaveFile::~WaveFile()
{
	if(wData) delete wData;
}



bool WaveFile::Error(AnsiString message)
{
	Application->MessageBox(message.c_str(),"Error",MB_OK);

	if(wData)
	{
		delete wData;
		wData=NULL;
	}

	return false;
}



bool WaveFile::Load(AnsiString filename)
{
	FILE *file;
	int i,j,n,ptr,length_bytes;

	file=fopen(filename.c_str(),"rb");

	if(!file) return false;

	fseek(file,0,SEEK_END);

	wSize=ftell(file);

	fseek(file,0,SEEK_SET);

	if(wData) free(wData);

	wData=new unsigned char[wSize];

	fread(wData,wSize,1,file);
	fclose(file);

	if(memcmp(wData,"RIFF",4)) return Error("RIFF chunk not found");

	if(memcmp(wData+8,"WAVEfmt ",8)) return Error("WAVEfmt chunk not found");

	if(RdWord(wData+20)>1) return Error("Compressed files are not supported");

	wChannels  =RdWord (wData+22);
	wSampleRate=RdDWord(wData+24);
	wBits      =RdWord (wData+34);

	if(wBits!=8&&wBits!=16&&wBits!=24) return Error("Only 8-, 16-, and 24-bit files supported");

	ptr=36;

	while(ptr<wSize)
	{
		if(!memcmp(wData+ptr,"data",4)) break;

		++ptr;
	}

	if(ptr>=wSize) return Error("DATA chunk not found");

	length_bytes=RdDWord(wData+ptr+4);

	wOffset=ptr+8;

	wLengthSamples=length_bytes/(wBits/8)/wChannels;

	return true;
}



bool WaveFile::SaveSelection(AnsiString filename,int start,int end)
{
	FILE *file;
	unsigned char* out;
	int i,ps,pd,len;

	if(!wData) return false;

	file=fopen(filename.c_str(),"wb");

	if(!file) return false;

	pd=44;
	ps=wOffset+start*(wBits/8)*wChannels;
	len=(end-start)*(wBits/8)*wChannels;

	out=(unsigned char*)malloc(44+len);

	memcpy (out,"RIFF",4);			//file id
	WrDWord(&out[4],len+44-8);		//file size-8
	memcpy (&out[8],"WAVEfmt ",8);	//format id
	WrDWord(&out[16],16);			//header size
	WrWord (&out[20],1);			//uncompressed PCM
	WrWord (&out[22],wChannels);	//channels
	WrDWord(&out[24],wSampleRate);	//sample rate
	WrDWord(&out[28],wSampleRate*wChannels*(wBits/8));	//byterate (sample rate*channels*bytes per sample)
	WrDWord(&out[32],wBits/8*wChannels);	//channels*bytes per sample
	WrDWord(&out[34],wBits);		//bits per sample
	memcpy (&out[36],"data",4);		//data id
	WrDWord(&out[40],len);			//data size in bytes

	for(i=0;i<len;++i) out[pd++]=wData[ps++];

	fwrite(out,len+44,1,file);
	fclose(file);

	free(out);

	return true;
}



int WaveFile::GetLength(void)
{
	return wLengthSamples;
}



int WaveFile::GetSampleRate(void)
{
	return wSampleRate;
}



int WaveFile::GetSample(int ptr)
{
	int chn,smp;

	if(ptr<0||ptr>=wLengthSamples) return 0;

	smp=0;

	switch(wBits)
	{
	case 8:
		{
			ptr=wOffset+ptr*wChannels;

			for(chn=0;chn<wChannels;++chn) smp+=(wData[ptr++]-128)*256;
		}
		break;

	case 16:
		{
			ptr=wOffset+ptr*2*wChannels;

			for(chn=0;chn<wChannels;++chn)
			{
				smp+=(signed short)(wData[ptr+0]+wData[ptr+1]*256);
				ptr+=2;
			}
		}
		break;

	case 24:
		{
			ptr=wOffset+ptr*3*wChannels;

			for(chn=0;chn<wChannels;++chn)
			{
				smp+=(signed short)(wData[ptr+1]+wData[ptr+2]*256);
				ptr+=3;
			}
		}
		break;
	}

	return smp/wChannels;
}
